// =============================
// Command processor
//

void EvalCommand(char c, int s)
{
    
      switch (CmdState[s]) {
        case CMD_ST_IDLE:
          if (ValidateCommand(c)) {
            CurCmd[s] = c;
            CmdIndex[s] = 0;
            CmdHasArg[s] = false;
            CmdState[s] = CMD_ST_CMD;
          }
          break;
        
        case CMD_ST_CMD:
          if (c == '=') {
            CmdArg[s] = 0;
            CmdState[s] = CMD_ST_ARG;
            CmdHasArg[s] = true;
          } else if (c == TERM_CHAR) {
            ProcessCommand(s);
            CmdState[s] = CMD_ST_IDLE;
          } else if ((c >= '0') && (c <= '9')) {
            CmdIndex[s] = CmdIndex[s]*10 + (c - '0');
          }
          break;
        
        case CMD_ST_ARG:
          if (c == TERM_CHAR) {
            ProcessCommand(s);
            CmdState[s] = CMD_ST_IDLE;
          } else if ((c >= '0') && (c <= '9')) {
            CmdArg[s] = CmdArg[s]*10 + (c - '0');
          }
          break;

        default:
          CmdState[s] = CMD_ST_IDLE;
      }
}


boolean ValidateCommand(char c)
{
  switch (c) {
    case 'H':
    case 'L':
    case 'T':
    case 'S':
    case 'V':
    case 'Z':
      return true;
      break;
    
    default:
      return false;
  }
}


void ProcessCommand(int s)
{
  switch (CurCmd[s]) {
    case 'H':
      CommandHelp(s);
      break;

    case 'L':
      if (CmdHasArg[s]) {
        log_count = CmdArg[s] / UPDATE_MSEC;
        cur_log_count = log_count;
        log_enable[s] = (log_count != 0);
      } else {
        PrintResponse(s, log_count*UPDATE_MSEC);
      }
      break;
    
    case 'T':
      if (CmdIndex[s] < NUM_TEMPS) {
        PrintFloat(s, cur_temp[CmdIndex[s]]);
      }
      break;

    case 'S':
      if (CmdIndex[s] < NUM_TEMPS) {
        PrintResponse(s, max_status[CmdIndex[s]]);
      }
      break;

    case 'V':
     if (s == 0) {
        Serial.print("V = ");
        Serial.println(VERSION_STRING);
      } else {
        SerialBT.print("V = ");
        SerialBT.println(VERSION_STRING);
      }
      break;

    case 'Z':
      zeroT = prevT;
      break;
  }
}


void PrintResponse(int src, int rsp)
{
  if (src == 0) {
    Serial.print(CurCmd[src]);
    Serial.print(" ");
    Serial.print(CmdIndex[src]);
    Serial.print(" = ");
    Serial.println(rsp);
  } else {
    SerialBT.print(CurCmd[src]);
    SerialBT.print(" ");
    SerialBT.print(CmdIndex[src]);
    SerialBT.print(" = ");
    SerialBT.println(rsp);
  }
}


void PrintFloat(int src, float rsp)
{
  if (src == 0) {
    Serial.print(CurCmd[src]);
    Serial.print(" ");
    Serial.print(CmdIndex[src]);
    Serial.print(" = ");
    Serial.println(rsp);
  } else {
    SerialBT.print(CurCmd[src]);
    SerialBT.print(" ");
    SerialBT.print(CmdIndex[src]);
    SerialBT.print(" = ");
    SerialBT.println(rsp);
  }
}

void CommandHelp(int src)
{
  if (src == 0) {
    Serial.print(help_string);
    Serial.print("Program Version ");
    Serial.println(VERSION_STRING);
  } else {
    SerialBT.print(help_string);
    SerialBT.print("Program Version ");
    SerialBT.println(VERSION_STRING);
  }
}
